#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>
#include <libavformat/avformat.h>
#include <libavcodec/avcodec.h>
#include <libavformat/avio.h>
#include <libavdevice/avdevice.h>
#include <libswscale/swscale.h>


void
print_usage()
{
	printf("Usage: video_cmp pattern.avi checked_video.avi\n");
}

double
video_cmp(AVFrame *pattern, AVFrame *checked_video, int w, int h)
{
	u_int8_t *data1 = pattern->data[0], *data2 = checked_video->data[0];
	int i, j, r_count, cmp_count, no_cmp_count, r1, r2;
	float res;
	r_count = cmp_count = no_cmp_count = 0;
	for (j = 0; j < h; j++)
		for (i = 0; i < w; i++) {
			r1 = data1[0] > 155 && data1[1] < data1[0] && data1[2] < data1[0];
			r2 = data2[0] > 155 && data2[1] < data2[0] && data2[2] < data2[0];
			if (r1)
				r_count++;
			if (r1 && r2)
				cmp_count++;
			if (r2 && !r1)
				no_cmp_count++;
			data1 += 3;
			data2 += 3;
		}
	if (r_count > 0)
		res = (double)(cmp_count + no_cmp_count) / r_count;
	else
		res = 0.0f;
	//printf("pattern features :%d\n matched features :%d\n excess features : %d\n", r_count, cmp_count, no_cmp_count);
	return res;
}

int
main(int argc, char *argv[])
{
	AVInputFormat *input_fmt1, *input_fmt2;
	AVFormatContext *fmt_ctx1, *fmt_ctx2;
	AVFormatParameters params;
	AVPacket packet1, packet2;
	AVFrame *frame1, *frame2, *rgbframe1, *rgbframe2;
	struct SwsContext *sws_ctx;
	struct SwsContext *sws_rgb;
	double quota;
	int i, err, count;
	int video_idx1, audio_idx1, video_idx2, audio_idx2;
	int h1, w1, h2, w2;
	int pix_fmt1, pix_fmt2, bufsize;

	u_int8_t *buf1, *buf2;
	//u_int8_t audio_buffer[AVCODEC_MAX_AUDIO_FRAME_SIZE];
	char *input_file1, *input_file2;

	buf1 = buf2 = NULL;
	frame1 = rgbframe1 = frame2 = rgbframe2 = NULL;
	video_idx1 = audio_idx1 = video_idx2 = audio_idx2 = -1;
	input_fmt1 = input_fmt2 = NULL;
	sws_ctx = sws_rgb = NULL;

	count = 0;
	quota = 0.0;

	memset(&params, 0, sizeof(params));
	params.width  = 0;
	params.height = 0;
	params.time_base = (AVRational){ 1, 5 };

	av_log_set_level(AV_LOG_DEBUG);

	avcodec_register_all();
	avdevice_register_all();
	av_register_all();



	if (argc < 3 || *argv[0] == '\0') {
		print_usage();
		exit(1);
	}

	input_file1 = argv[1];
	input_file2 = argv[2];


	err = av_open_input_file(&fmt_ctx1, input_file1, input_fmt1, 0, &params);

	if (err < 0) {
		printf("can't open file %s\n", input_file1);
		exit(1);
	}

	err = av_open_input_file(&fmt_ctx2, input_file2, input_fmt2, 0, &params);

	if (err < 0) {
		printf("can't open file %s\n", input_file2);
		exit(1);
	}


	err = av_find_stream_info(fmt_ctx1);

	if (err < 0) {
		printf("can't find stream info\n");
		exit(1);
	}

	err = av_find_stream_info(fmt_ctx2);

	if (err < 0) {
		printf("can't find stream info\n");
		exit(1);
	}

	dump_format(fmt_ctx1, 0, input_file1, 0);
	dump_format(fmt_ctx2, 0, input_file2, 0);

	for (i = 0; i < fmt_ctx1->nb_streams; i++) {
		struct AVStream *stream;
		struct AVCodec *codec;
		int id;

		stream = fmt_ctx1->streams[i];
		id = stream->codec->codec_id;

		switch (stream->codec->codec_type) {
		case CODEC_TYPE_VIDEO:
			if (video_idx1 >= 0)
				break;
			if ((codec = avcodec_find_decoder(id)) == NULL) {
				printf("can't find codec for video stream %d\n", i);
			} else if ((err = avcodec_open(stream->codec, codec)) < 0) {
				printf("can't open codec %s for stream %d\n", codec->name, i);
			} else {
				video_idx1 = i;
			}
			break;
		case CODEC_TYPE_AUDIO:
			if (audio_idx1 >= 0)
				break;
			if ((codec = avcodec_find_decoder(id)) == NULL) {
				printf("can't find codec for audio stream %d\n", i);
			} else if ((err = avcodec_open(stream->codec, codec)) < 0) {
				printf("can't open codec %s for stream %d\n", codec->name, i);
			} else {
				audio_idx1 = i;
				stream->codec->workaround_bugs = 1;
			}
			break;
		default:
			break;
		}
	}

	for (i = 0; i < fmt_ctx2->nb_streams; i++) {
		struct AVStream *stream;
		struct AVCodec *codec;
		int id;

		stream = fmt_ctx2->streams[i];
		id = stream->codec->codec_id;

		switch (stream->codec->codec_type) {
		case CODEC_TYPE_VIDEO:
			if (video_idx2 >= 0)
				break;
			if ((codec = avcodec_find_decoder(id)) == NULL) {
				printf("can't find codec for video stream %d\n", i);
			} else if ((err = avcodec_open(stream->codec, codec)) < 0) {
				printf("can't open codec %s for stream %d\n", codec->name, i);
			} else {
				video_idx2 = i;
			}
			break;
		case CODEC_TYPE_AUDIO:
			if (audio_idx2 >= 0)
				break;
			if ((codec = avcodec_find_decoder(id)) == NULL) {
				printf("can't find codec for audio stream %d\n", i);
			} else if ((err = avcodec_open(stream->codec, codec)) < 0) {
				printf("can't open codec %s for stream %d\n", codec->name, i);
			} else {
				audio_idx2 = i;
				stream->codec->workaround_bugs = 1;
			}
			break;
		default:
			break;
		}
	}

	if (audio_idx2 < 0 && video_idx2 < 0) {
		printf("no audio or video stream present\n");
		exit(1);
	}

	frame1 = avcodec_alloc_frame();
	if (frame1 == NULL)
		goto err;

	frame2 = avcodec_alloc_frame();
	if (frame2 == NULL)
		goto err;

	rgbframe1 = avcodec_alloc_frame();
	if (rgbframe1 == NULL)
		goto err;

	rgbframe2 = avcodec_alloc_frame();
	if (rgbframe2 == NULL)
		goto err;

	pix_fmt1 = fmt_ctx1->streams[video_idx1]->codec->pix_fmt;
	h1 = fmt_ctx1->streams[video_idx1]->codec->height;
	w1 = fmt_ctx1->streams[video_idx1]->codec->width;

	pix_fmt2 = fmt_ctx2->streams[video_idx2]->codec->pix_fmt;
	h2 = fmt_ctx2->streams[video_idx2]->codec->height;
	w2 = fmt_ctx2->streams[video_idx2]->codec->width;

	if (h1 != h2 || w1 != w2) {
		printf("videos are of different sizes\n");
		goto err;
	}

	bufsize = avpicture_get_size(PIX_FMT_RGB24, w1, h1);

	if (bufsize > 0) {
		buf1 = av_malloc(bufsize);
		avpicture_fill((AVPicture *)rgbframe1, buf1, PIX_FMT_RGB24, w1, h1);
		buf2 = av_malloc(bufsize);
		avpicture_fill((AVPicture *)rgbframe2, buf2, PIX_FMT_RGB24, w2, h2);
	} else {
		printf("can't get bufsize for pixel format %s\n",
		       avcodec_get_pix_fmt_name(PIX_FMT_RGB24));
		exit(1);
	}

	sws_ctx = sws_getContext(w1, h1, pix_fmt1, w1, h1, PIX_FMT_RGB24,
				 SWS_BICUBIC, NULL, NULL, NULL);

	sws_rgb = sws_getContext(w1, h1, PIX_FMT_RGB24, w1, h1, PIX_FMT_YUV420P,
			 SWS_BICUBIC, NULL, NULL, NULL);

	if (sws_ctx == NULL || sws_rgb == NULL) {
		printf("can't alloc sws context\n");
		exit(1);
	}

	while ((av_read_frame(fmt_ctx1, &packet1) >= 0) && (av_read_frame(fmt_ctx2, &packet2) >= 0)) {
		int frame_ready = 0;
		AVCodecContext *codec_ctx;

		if (video_idx1 >= 0 && packet1.stream_index == video_idx1) {
			codec_ctx = fmt_ctx1->streams[video_idx1]->codec;

			err = avcodec_decode_video(codec_ctx, frame1,
					&frame_ready, packet1.data, packet1.size);

			if (err < 0) {
				printf("can't decode video packet\n");
			} else if (frame_ready) {
				sws_scale(sws_ctx, frame1->data, frame1->linesize,
					  0, h1, rgbframe1->data, rgbframe1->linesize);
			}

		}

		if (video_idx2 >= 0 && packet2.stream_index == video_idx2) {
					codec_ctx = fmt_ctx2->streams[video_idx2]->codec;

			err = avcodec_decode_video(codec_ctx, frame2,
					&frame_ready, packet2.data, packet2.size);

			if (err < 0) {
				printf("can't decode video packet\n");
			} else if (frame_ready) {
				sws_scale(sws_ctx, frame2->data, frame2->linesize,
					  0, h1, rgbframe2->data, rgbframe2->linesize);
			}

		}
		quota += video_cmp(rgbframe1, rgbframe2, w1, h1);

		count++;
		av_free_packet(&packet1);
		av_free_packet(&packet2);

	}
	printf("Quit\n");

err:

	if (frame1 != NULL)
		av_free(frame1);
	if (rgbframe1 != NULL)
		av_free(rgbframe1);
	if (buf1 != NULL)
		av_free(buf1);

	if (frame2 != NULL)
		av_free(frame2);
	if (rgbframe2 != NULL)
		av_free(rgbframe2);
	if (buf2 != NULL)
		av_free(buf2);

	if (sws_ctx != NULL)
		sws_freeContext(sws_ctx);

	if (sws_rgb != NULL)
		sws_freeContext(sws_rgb);

	for (i = 0; i < fmt_ctx1->nb_streams; i++) {
		struct AVStream *stream;
		stream = fmt_ctx1->streams[i];
		avcodec_close(stream->codec);
	}

	for (i = 0; i < fmt_ctx2->nb_streams; i++) {
		struct AVStream *stream;
		stream = fmt_ctx2->streams[i];
		avcodec_close(stream->codec);
	}

	av_close_input_file(fmt_ctx1);
	av_close_input_file(fmt_ctx2);

	if (count > 0)
		quota /= count;
	quota *= 100;
	printf("----------------------------------------\n");
	if (quota < 80)
		printf("Схожесть небольшая %d процентов \n", (int)quota);
	else if (quota >= 80 || quota <= 110)
		printf("Схожесть хорошая %d процентов \n", (int)quota);
	else if (quota > 110)
		printf("Найдено больше движения чем на образце на  %d процентов \n", (int)quota - 100);
	printf("----------------------------------------\n");
	return (int)quota;
}
